package com.example.demo.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.department.service.DepartmentService;
import com.example.demo.directions.model.DirectionsEntity;
import com.example.demo.directions.service.DirectionsService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class DirectionsTests {
    @Autowired
    private DirectionsService directionsService;
    @Autowired
    private DepartmentService departmentService;

    private DepartmentEntity dep1;
    private DepartmentEntity dep2;

    private DirectionsEntity direct1;
    private DirectionsEntity direct2;
    private DirectionsEntity direct3;

    @BeforeEach
    void createData() {
        removeData();

        dep1 = departmentService.create(new DepartmentEntity("GUM"));
        dep2 = departmentService.create(new DepartmentEntity("IST"));

        direct1 = directionsService
                .create(new DirectionsEntity("09.03.23", "Программная инженерия", dep2, "Инф"));
        direct2 = directionsService.create(
                new DirectionsEntity("09.04.24", "Про", dep1, "Рус"));
        direct3 = directionsService.create(
                new DirectionsEntity("09.05.25", "Прогdcя", dep1, "Мат"));
    }

    @AfterEach
    void removeData() {
        directionsService.getAll(0L).forEach(item -> directionsService.delete(item.getId()));
        departmentService.getAll().forEach(item -> departmentService.delete(item.getId()));
    }

    @Test
    @Order(1)
    void createTest() {
        Assertions.assertEquals(3, directionsService.getAll(0L).size());
        Assertions.assertEquals(2, directionsService.getAll(dep1.getId()).size());
        Assertions.assertEquals(1, directionsService.getAll(dep2.getId()).size());
    }

    @Test
    @Order(2)
    void createNotUniqueTest() {
        final DirectionsEntity nonUniquedirection = new DirectionsEntity("09.04.24", "Про", new DepartmentEntity("ИЯ"),
                "Рус");
        Assertions.assertThrows(IllegalArgumentException.class, () -> directionsService.create(nonUniquedirection));
    }

    @Test
    @Order(3)
    void createNullableTest() {
        final DirectionsEntity nullabledirection = new DirectionsEntity(null, null, null, null);
        Assertions.assertThrows(
                DataIntegrityViolationException.class,
                () -> directionsService.create(nullabledirection));
    }

    @Test
    @Order(4)
    void updateTest() {
        final String testCode = "TESTCODE";
        final String testName = "TESTNAME";
        final DepartmentEntity testDep = new DepartmentEntity("TESTDEP");
        final String testThin = "TESTTHIN";
        final DirectionsEntity entity = directionsService.get(direct1.getId());
        final String oldCode = entity.getCode();
        final String oldName = entity.getName();
        final DepartmentEntity oldDep = entity.getDepartment();
        final String oldThin = entity.getThings();
        final DirectionsEntity testEntity = new DirectionsEntity(testCode, testName, testDep, testThin);
        DirectionsEntity newEntity = directionsService.update(direct1.getId(), testEntity);
        Assertions.assertEquals(3, directionsService.getAll(0L).size());
        Assertions.assertEquals(newEntity, direct1);
        Assertions.assertEquals(testCode, newEntity.getCode());
        Assertions.assertNotEquals(oldCode, newEntity.getCode());

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());

        Assertions.assertEquals(testThin, newEntity.getThings());
        Assertions.assertNotEquals(oldThin, newEntity.getThings());
    }

    @Test
    @Order(5)
    void deleteTest() {
        directionsService.delete(direct2.getId());
        Assertions.assertEquals(2, directionsService.getAll(0L).size());

        final DirectionsEntity newEntity = directionsService
                .create(new DirectionsEntity("test1", "test2",
                        new DepartmentEntity("test3"), "test4"));
        Assertions.assertEquals(3, directionsService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
